/*

  Programma Eq_parabolica.ccp:
  Programma che integra una equazione parabolica
  con uno schema di Runge-Kutta al secondo ordine.

  d f / dt = - v * d^2 f / dx^2

  in un intervallo [a,b] con condizioni al bordo
  di tipo:
  1) Dirichlet
  2) Neumann
  3) Periodiche

  Il programma legge i dati di input dal file: input.data.

 */

#include <iostream>
#include <fstream>
#include <cmath>
#include "VarX.h"
using namespace std;


/*

  Programma principale

*/

int main()
{

  // Lettura dati iniziali dal file
  ifstream finp;
  finp.open( "input.data" );

  // Lettura numero di punti griglia
  long int numPoints;
  finp >> numPoints;

  // Lettura passo temporale della simulazione
  double dt;
  finp >> dt;

  // Lettura durata totale della simulazione
  double tEnd;
  finp >> tEnd;

  // Lettura intervallo tra le stampe in output
  double tPrint;
  finp >> tPrint;

  // Lettura parametro nu
  double nu;
  finp >> nu;

  // Lettura parametro k
  long int k;
  finp >> k;

  // Chiusura file di input
  finp.close();


  // Variabili per griglia spaziale
  double pi = acos( -1.0 );
  double a, b;
  a = -pi;
  b = +pi;


  // Variabili per ciclo temporale
  long nEnd   = long( tEnd / dt );
  long nPrint = long( tPrint / dt );


  // Costruzione dei vettori di lavoro
  double *x   = new double[numPoints];
  double *f   = new double[numPoints];
  double *fs  = new double[numPoints];
  double *d2f = new double[numPoints];


  // Definizione dei punti griglia e della condizione iniziale
  double dx = ( b - a ) / ( numPoints - 1 );
  for ( long int i = 0; i < numPoints; i++ )
    {
      x[i] = a + i * dx;
      //f[i] = 1.0 + cos( k * x[i] );
      f[i] = 0.0;
    }
  f[(numPoints-1)/2] = 2.0;

  // Oggetto della classe VarX per calcolo delle derivate
  VarX ff = VarX( numPoints, f, 1, dx );

  double tCurr = 0.0, tNext;

  // Ciclo temporale

  for ( long int nt = 1; nt <= nEnd; nt++ )
    {

      tCurr = ( nt - 1 ) * dt;
      tNext = nt * dt;

      // Primo step del Runge-Kutta

      // Calcolo derivata seconda
      ff.getDerXX( d2f );

      // Calcolo variabile al passo intermedio
      for ( long int i = 0; i < numPoints; i++ )
	fs[i] = f[i] + 0.5 * dt * nu * d2f[i];

      // Valore del campo al passo intermedio
      ff.setValue( fs );


      // Secondo step del Runge-Kutta

      // Calcolo della derivata seconda
      ff.getDerXX( d2f );

      // Calcolo campo al tempo successivo
      for ( long int i = 0; i < numPoints; i++ )
	f[i] = f[i] + dt * nu * d2f[i];


      // Valore del campo al passo finale
      f[(numPoints-1)/2] = 2.0;
      ff.setValue( f );


      // Stampa ogni Tprint intervalli di stampa

      if ( ( nt % nPrint ) == 0 )
	{
	  cout << "set title 't = " << tNext << "'" << endl;
	  cout << "set xlabel 'x'" << endl;
	  cout << "set ylabel 'f'" << endl;
	  cout << "set xrange [" << a << ":" << b << "]" << endl;
	  cout << "plot '-' using 1:2 notitle with lines" << endl;
	  for ( long int i = 0; i < numPoints; i++ )
	    cout << x[i] << "   " << f[i] << endl;
	  cout << "e" << endl;
	}

    }

}
