/*

  File VarX.ccp:

  Calcola le derivate prime e seconde di un campo f(x) tramite uno schema alle differenze
  finite del secondo ordine con vari tipi di condizioni al bordo.

*/


#include "VarX.h"


// Costruttore della classe VarX

VarX::VarX( long int n, double * y0, int bc, double dx )
{
  nx = n;
  y = new double[nx];
  boundCond = bc;
  setValue( y0 );
  deltaX = dx;
  dueDeltaX = 2.0 * deltaX;
  deltaX2 = dx * dx;
}


// Distruttore della classe

VarX::~VarX( )
{
  delete [] y;
}


// Funzione setValue: inizializza il vettore delle incognite con la condizione iniziale

void VarX::setValue( double * y0 )
{
  for ( long int i = 0; i < nx; i++ ) y[i] = y0[i];
}


// Funzione getDerX: restituisce il valore della derivata della variabile al tempo corrente
// tenendo conto delle condizioni al bordo

void VarX::getDerX( double * sol )
{
  // Punti interni al dominio

  for ( long int i = 1; i < nx - 1; i++ )
    {
      sol[i] = ( y[i+1] - y[i-1] ) / dueDeltaX;
    }

  // Condizioni al bordo
  if ( boundCond == 0 )     // Condizioni di tipo Dirichlet
    {
      // Funzione nulla sul bordo,
      // derivata calcolata dai punti interni con formule decentrate
      sol[0] = ( 4.0 * y[1] - y[2] ) / dueDeltaX;
      sol[nx-1] = ( - 4.0 * y[nx-2] + y[nx-3] ) / dueDeltaX;
    }
  if ( boundCond == 1 )    // Condizioni di tipo Neumann
    {
      // derivata nulla sul bordo
      sol[0] = 0.0;
      sol[nx-1] = 0.0;
    }
  if ( boundCond == 2 )   // Condizioni periodiche
    {
      sol[0] = ( y[1] - y[nx-2] ) / dueDeltaX;
      sol[nx-1] = ( y[1] - y[nx-2] ) / dueDeltaX;
    }
}


// Funzione getDerXX: restituisce il valore della derivata seconda della variabile al tempo corrente
// tenendo conto delle condizioni al bordo

void VarX::getDerXX( double * sol )
{
  // Punti interni al dominio

  for ( long int i = 1; i < nx - 1; i++ )
    {
      sol[i] = ( y[i+1] - 2.0 * y[i] + y[i-1] ) / deltaX2;
    }

  // Condizioni al bordo
  if ( boundCond == 0 )     // Condizioni di tipo Dirichlet
    {
      // Funzione nulla sul bordo,
      // derivata calcolata dai punti interni con formule decentrate
      sol[0] = ( - 5.0 * y[1] + 4.0 * y[2] -y[3] ) / deltaX2;
      sol[nx-1] = ( - 5.0 * y[nx-2] + 4.0 * y[nx-3] -y[nx-4] ) / deltaX2;
    }
  if ( boundCond == 1 )    // Condizioni di tipo Neumann
    {
      // derivata nulla sul bordo
      sol[0] = ( -7.0 * y[0] + 8.0 * y[1] - y[2] ) / ( 2.0 * deltaX2 );
      sol[nx-1] = ( -7.0 * y[nx-1] + 8.0 * y[nx-2] - y[nx-3] ) / ( 2.0 * deltaX2 );
    }
  if ( boundCond == 2 )   // Condizioni periodiche
    {
      sol[0] = ( y[1] - 2.0 * y[0] + y[nx-2] ) / deltaX2;
      sol[nx-1] = ( y[1] - 2.0 * y[nx-1] + y[nx-2] ) / deltaX2;
    }

}
